/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.apache.dubbo.rpc.protocol.tri;

import io.netty.handler.codec.http2.Http2Headers;

import java.util.Iterator;
import java.util.Map;

public class Http2HeaderMeta implements Metadata {
    private final Http2Headers headers;

    public Http2HeaderMeta(Http2Headers headers) {
        this.headers = headers;
    }

    @Override
    public Metadata put(CharSequence key, CharSequence value) {
        headers.set(key, value);
        return this;
    }

    @Override
    public CharSequence get(CharSequence key) {
        return headers.get(key);
    }

    @Override
    public boolean contains(CharSequence key) {
        return headers.contains(key);
    }

    @Override
    public boolean remove(CharSequence key) {
        return headers.remove(key);
    }

    @Override
    public Iterator<Map.Entry<CharSequence, CharSequence>> iterator() {
        return headers.iterator();
    }
}
